@@ -257,13 +257,13 @@ void ProcessWork(ares_channel_t *channel,
   fd_set readers, writers;
 
 #ifndef CARES_SYMBOL_HIDING
-  struct timeval tv_begin  = ares__tvnow();
-  struct timeval tv_cancel = tv_begin;
+  ares_timeval_t tv_begin  = ares__tvnow();
+  ares_timeval_t tv_cancel = tv_begin;
 
   if (cancel_ms) {
     if (verbose) std::cerr << "ares_cancel will be called after " << cancel_ms << "ms" << std::endl;
-    tv_cancel.tv_sec  += (cancel_ms / 1000);
-    tv_cancel.tv_usec += ((cancel_ms % 1000) * 1000);
+    tv_cancel.sec  += (cancel_ms / 1000);
+    tv_cancel.usec += ((cancel_ms % 1000) * 1000);
   }
 #else
   if (cancel_ms) {
@@ -274,8 +274,8 @@ void ProcessWork(ares_channel_t *channel,
 
   while (true) {
 #ifndef CARES_SYMBOL_HIDING
-    struct timeval  tv_now = ares__tvnow();
-    struct timeval  tv_remaining;
+    ares_timeval_t  tv_now = ares__tvnow();
+    ares_timeval_t  atv_remaining;
 #endif
     struct timeval  tv;
     struct timeval *tv_select;
@@ -305,15 +305,21 @@ void ProcessWork(ares_channel_t *channel,
 #ifndef CARES_SYMBOL_HIDING
     if (cancel_ms) {
       unsigned int remaining_ms;
-      ares__timeval_remaining(&tv_remaining,
+      ares__timeval_remaining(&atv_remaining,
                               &tv_now,
                               &tv_cancel);
-      remaining_ms = (unsigned int)((tv_remaining.tv_sec * 1000) + (tv_remaining.tv_usec / 1000));
+
+      remaining_ms = (unsigned int)((atv_remaining.sec * 1000) + (atv_remaining.usec / 1000));
       if (remaining_ms == 0) {
         if (verbose) std::cerr << "Issuing ares_cancel()" << std::endl;
         ares_cancel(channel);
         cancel_ms = 0; /* Disable issuing cancel again */
       } else {
+        struct timeval tv_remaining;
+
+        tv_remaining.tv_sec = atv_remaining.sec;
+        tv_remaining.tv_usec = (int)atv_remaining.usec;
+
         /* Recalculate proper timeout since we also have a cancel to wait on */
         tv_select = ares_timeout(channel, &tv_remaining, &tv);
       }
@@ -574,15 +580,16 @@ void MockServer::ProcessPacket(ares_socket_t fd, struct sockaddr_storage *addr,
   }
   int rrtype = DNS_QUESTION_TYPE(question);
 
+  std::vector<byte> req(data, data + len);
+  std::string reqstr = PacketToString(req);
   if (verbose) {
-    std::vector<byte> req(data, data + len);
-    std::cerr << "received " << (fd == udpfd_ ? "UDP" : "TCP") << " request " << PacketToString(req)
+    std::cerr << "received " << (fd == udpfd_ ? "UDP" : "TCP") << " request " << reqstr
               << " on port " << (fd == udpfd_ ? udpport_ : tcpport_)
               << ":" << getaddrport(addr) << std::endl;
     std::cerr << "ProcessRequest(" << qid << ", '" << namestr
               << "', " << RRTypeToString(rrtype) << ")" << std::endl;
   }
-  ProcessRequest(fd, addr, addrlen, qid, namestr, rrtype);
+  ProcessRequest(fd, addr, addrlen, reqstr, qid, namestr, rrtype);
 
 }
 
@@ -651,11 +658,17 @@ std::set<ares_socket_t> MockServer::fds() const {
 }
 
 
-void MockServer::ProcessRequest(ares_socket_t fd, struct sockaddr_storage* addr, ares_socklen_t addrlen,
+void MockServer::ProcessRequest(ares_socket_t fd, struct sockaddr_storage* addr,
+                                ares_socklen_t addrlen, const std::string &reqstr,
                                 int qid, const std::string& name, int rrtype) {
   // Before processing, let gMock know the request is happening.
   OnRequest(name, rrtype);
 
+  // If we are expecting a specific request then check it matches here.
+  if (expected_request_.length() > 0) {
+    ASSERT_EQ(expected_request_, reqstr);
+  }
+
   if (reply_.size() == 0) {
     return;
   }
@@ -728,9 +741,9 @@ MockChannelOptsTest::MockChannelOptsTest(int count,
   opts.tcp_port = server_.tcpport();
   optmask |= ARES_OPT_TCP_PORT;
 
-  // If not already overridden, set short-ish timeouts.
   if (!(optmask & (ARES_OPT_TIMEOUTMS|ARES_OPT_TIMEOUT))) {
-    opts.timeout = 1500;
+    // Reduce timeouts significantly to shorten test times.
+    opts.timeout = 250;
     optmask |= ARES_OPT_TIMEOUTMS;
   }
   // If not already overridden, set 3 retries.
@@ -824,8 +837,8 @@ void MockEventThreadOptsTest::ProcessThread() {
 
 #ifndef CARES_SYMBOL_HIDING
   bool has_cancel_ms = false;
-  struct timeval tv_begin;
-  struct timeval tv_cancel;
+  ares_timeval_t tv_begin;
+  ares_timeval_t tv_cancel;
 #endif
 
   mutex.lock();
@@ -834,14 +847,14 @@ void MockEventThreadOptsTest::ProcessThread() {
     int nfds = 0;
     fd_set readers;
 #ifndef CARES_SYMBOL_HIDING
-    struct timeval  tv_now = ares__tvnow();
-    struct timeval  tv_remaining;
+    ares_timeval_t tv_now = ares__tvnow();
+    ares_timeval_t atv_remaining;
     if (cancel_ms_ && !has_cancel_ms) {
       tv_begin  = ares__tvnow();
       tv_cancel = tv_begin;
       if (verbose) std::cerr << "ares_cancel will be called after " << cancel_ms_ << "ms" << std::endl;
-      tv_cancel.tv_sec  += (cancel_ms_ / 1000);
-      tv_cancel.tv_usec += ((cancel_ms_ % 1000) * 1000);
+      tv_cancel.sec  += (cancel_ms_ / 1000);
+      tv_cancel.usec += ((cancel_ms_ % 1000) * 1000);
       has_cancel_ms = true;
     }
 #else
@@ -866,10 +879,10 @@ void MockEventThreadOptsTest::ProcessThread() {
 #ifndef CARES_SYMBOL_HIDING
     if (has_cancel_ms) {
       unsigned int remaining_ms;
-      ares__timeval_remaining(&tv_remaining,
+      ares__timeval_remaining(&atv_remaining,
                               &tv_now,
                               &tv_cancel);
-      remaining_ms = (unsigned int)((tv_remaining.tv_sec * 1000) + (tv_remaining.tv_usec / 1000));
+      remaining_ms = (unsigned int)((atv_remaining.sec * 1000) + (atv_remaining.usec / 1000));
       if (remaining_ms == 0) {
         if (verbose) std::cerr << "Issuing ares_cancel()" << std::endl;
         ares_cancel(channel_);
@@ -1076,6 +1089,23 @@ void SearchCallback(void *data, int status, int timeouts,
   if (verbose) std::cerr << "SearchCallback(" << *result << ")" << std::endl;
 }
 
+void SearchCallbackDnsRec(void *data, ares_status_t status, size_t timeouts,
+                          const ares_dns_record_t *dnsrec) {
+  EXPECT_NE(nullptr, data);
+  SearchResult* result = reinterpret_cast<SearchResult*>(data);
+  unsigned char *abuf = NULL;
+  size_t alen = 0;
+  result->done_ = true;
+  result->status_ = (int)status;
+  result->timeouts_ = (int)timeouts;
+  if (dnsrec != NULL) {
+    ares_dns_write(dnsrec, &abuf, &alen);
+  }
+  result->data_.assign(abuf, abuf + alen);
+  ares_free_string(abuf);
+  if (verbose) std::cerr << "SearchCallbackDnsRec(" << *result << ")" << std::endl;
+}
+
 std::ostream& operator<<(std::ostream& os, const NameInfoResult& result) {
   os << '{';
   if (result.done_) {
